@@ -26,6 +26,7 @@ BSgenome_datapkgs <- c(
   "BSgenome.Cfamiliaris.UCSC.canFam2",
   "BSgenome.Cfamiliaris.UCSC.canFam3",
   "BSgenome.Cjacchus.UCSC.calJac3",
+  "BSgenome.Cjacchus.UCSC.calJac4",
   "BSgenome.Creinhardtii.JGI.v5.6",
   "BSgenome.Dmelanogaster.UCSC.dm2",
   "BSgenome.Dmelanogaster.UCSC.dm3",
@@ -48,8 +49,10 @@ BSgenome_datapkgs <- c(
   "BSgenome.Hsapiens.UCSC.hg19",
   "BSgenome.Hsapiens.NCBI.GRCh38",
   "BSgenome.Hsapiens.UCSC.hg38",
+  "BSgenome.Hsapiens.NCBI.T2T.CHM13v2.0",
   "BSgenome.Mdomestica.UCSC.monDom5",
   "BSgenome.Mfascicularis.NCBI.5.0",
+  "BSgenome.Mfascicularis.NCBI.6.0",
   "BSgenome.Mfuro.UCSC.musFur1",
   "BSgenome.Mmulatta.UCSC.rheMac2",
   "BSgenome.Mmulatta.UCSC.rheMac3",
@@ -93,6 +96,7 @@ MaskedBSgenome_datapkgs <- c(
   "BSgenome.Btaurus.UCSC.bosTau3.masked",
   "BSgenome.Btaurus.UCSC.bosTau4.masked",
   "BSgenome.Btaurus.UCSC.bosTau6.masked",
+  "BSgenome.Btaurus.UCSC.bosTau9.masked",
   "BSgenome.Cfamiliaris.UCSC.canFam2.masked",
   "BSgenome.Cfamiliaris.UCSC.canFam3.masked",
   "BSgenome.Dmelanogaster.UCSC.dm2.masked",
